/*
 * Copyright (C) 2016  (Salomon Sickert)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package explicit.nxt;

import explicit.NondetModel;
import explicit.Product;
import it.unimi.dsi.fastutil.ints.Int2IntMap;

/**
 * Make LTL product accessible as a Product
 */
public class LDBAProduct<M extends NondetModel> extends Product<M> {
    private final Int2IntMap invMap;

    private final SparseBitSet goalStates;

    public LDBAProduct(M productModel, M originalModel, Int2IntMap invMap, SparseBitSet goalStates) {
        super(productModel, originalModel);
        this.invMap = invMap;
        invMap.defaultReturnValue(-1);
        this.goalStates = goalStates;
    }

    @Override
    public int getAutomatonState(int productState) {
        return invMap.get(productState);
    }

    public SparseBitSet getGoalStates() {
        return goalStates;
    }

    @Override
    public int getModelState(int productState) {
        return invMap.get(productState);
    }
}
