/*
 * Copyright (C) 2016  (Salomon Sickert)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package explicit.nxt;

import explicit.MDP;
import it.unimi.dsi.fastutil.ints.Int2ObjectMap;
import prism.PrismException;

import java.util.BitSet;
import java.util.Collection;
import java.util.List;

public class LightAcceptingEC {

    /**
     * Use the given Büchi condition to identify goal states in the model. Restrict the search to the given preMECs.
     * @param model The model MDP
     * @param acceptance Generalised Büchi Acceptance
     * @param preMECs MEC candidates that need to be refined.
     * @return a set of states needed to be reached for acceptance
     */
    public static SparseBitSet findAcceptingECStates(MDP model, List<Int2ObjectMap<BitSet>> acceptance, Collection<LightMEC> preMECs) {
        SparseBitSet allAcceptingStates = new SparseBitSet();

        for (LightMEC mec : LightECComputer.computeMECs(model, preMECs)) {
            boolean allSuccess = true;

            for (Int2ObjectMap<BitSet> map : acceptance) {
                boolean indexSuccess = false;

                for (int i = mec.states.nextSetBit(0); i >= 0 && !indexSuccess; i = mec.states.nextSetBit(i + 1)) {
                    BitSet actions = map.get(i);

                    if (actions != null) {
                        indexSuccess = actions.intersects(mec.actions.get(i));
                    }
                }

                if (!indexSuccess) {
                    allSuccess = false;
                    break;
                }
            }

            if (allSuccess) {
                allAcceptingStates.or(mec.states);
            }
        }

        return allAcceptingStates;
    }

}
