/*
 * Copyright (C) 2016  (Salomon Sickert)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package explicit.nxt;

import explicit.PrismExplicit;
import parser.ast.ModulesFile;
import parser.ast.PropertiesFile;
import parser.ast.Property;
import prism.*;
import simulator.ModulesFileModelGenerator;

import java.io.File;

public class NxtCLI {

    private static void printHelp() {
        System.out.println("Usage: ./ProMoChiBA.sh model.nm property.pctl");
    }

    public static void main(String... args) throws Exception {
        try {
            PrismLog mainLog = new PrismPrintStreamLog(System.out);
            ModulesFile modulesFile = Prism.parseModelFile(new File(args[0]));
            modulesFile.setUndefinedConstants(null);
            PropertiesFile propertiesFile = Prism.parsePropertiesFile(modulesFile, new File(args[1]));
            propertiesFile.setUndefinedConstants(null);
            PrismExplicit pe = new PrismExplicit(mainLog, new PrismSettings());
            explicit.Model modelExpl = pe.buildModel(modulesFile, new ModulesFileModelGenerator(modulesFile, pe));

            for (Property prop : propertiesFile.properties) {
                System.out.println("-----------------------------------------------------");
                System.gc();
                pe.modelCheck(modelExpl, modulesFile, propertiesFile, prop.getExpression());
            }
        } catch (Exception ex) {
            printHelp();
            System.out.println();
            System.out.println("Oops, something went terribly wrong! Please have a look at the usage example.");
            System.out.println("Reason: " + ex.toString());
            ex.printStackTrace();
        }
    }
}